#include "musicplayer.h"
#include "config.h"
#include <QQmlContext>

#ifdef ADD_MPRIS
    #include <QDBusConnection>
    #include <QDBusConnectionInterface>
#endif

MusicPlayer::MusicPlayer (int& argc, char** argv) : QApplication (argc, argv), pView (nullptr)
{
    setOrganizationName ("UbPorts");
    setOrganizationDomain ("ubports.com");
    setApplicationName ("Music Player");

    #ifdef ADD_MPRIS
        QDBusConnection cConnection = QDBusConnection::sessionBus ();
        QDBusConnectionInterface *pInterface = cConnection.interface ();
        bool bMediaHub = pInterface->isServiceRegistered ("com.lomiri.MediaHub.Service");

        if (bMediaHub == false)
        {
            this->pMprisPlayer =  new MprisPlayer ();
            this->pMprisPlayer->setServiceName ("lomiri-music-app");
            this->pMprisPlayer->setSupportedUriSchemes (QStringList () << "file");
            this->pMprisPlayer->setCanQuit (false);
            this->pMprisPlayer->setCanRaise (false);
            this->pMprisPlayer->setCanSetFullscreen (false);
            this->pMprisPlayer->setHasTrackList (false);
            this->pMprisPlayer->setDesktopEntry ("lomiri-music-app");
            this->pMprisPlayer->setIdentity ("Lomiri Music App");
            this->pMprisPlayer->setCanControl (false);
            this->pMprisPlayer->setCanPlay (true);
            this->pMprisPlayer->setCanGoNext (true);
            this->pMprisPlayer->setCanGoPrevious (true);
            this->pMprisPlayer->setCanPause (true);
            this->pMprisPlayer->setCanSeek (false);
        }
    #endif
}

#ifdef ADD_MPRIS
    void MusicPlayer::updateMpris (bool bCanControl, int nStatus, bool bCanGoPrevious, bool bCanGoNext, QString sAlbum, QString sArt, QString sArtist, QString sTitle)
    {
        if (this->pMprisPlayer == nullptr)
        {
            return;
        }

        Mpris::PlaybackStatus eStatus = Mpris::Stopped;

        if (nStatus == 1)
        {
            eStatus = Mpris::Playing;
        }
        else if (nStatus == 2)
        {
            eStatus = Mpris::Paused;
        }

        this->pMprisPlayer->setCanControl (bCanControl);
        this->pMprisPlayer->setPlaybackStatus (eStatus);
        this->pMprisPlayer->setCanGoNext (bCanGoNext);
        this->pMprisPlayer->setCanGoPrevious (bCanGoPrevious);

        if (sAlbum != nullptr || sArt != nullptr || sArtist != nullptr || sTitle != nullptr)
        {
            QVariantMap pMetadata;

            if (sArtist != nullptr)
            {
                pMetadata.insert ("xesam:artist", QStringList() << sArtist);
            }

            if (sTitle != nullptr)
            {
                pMetadata.insert ("xesam:title", sTitle);
            }

            if (sAlbum != nullptr)
            {
                pMetadata.insert ("xesam:album", sAlbum);
            }

            if (sArt != nullptr)
            {
                pMetadata.insert ("mpris:artUrl", sArt);
            }

            this->pMprisPlayer->setMetadata (pMetadata);
        }
    }
#endif

MusicPlayer::~MusicPlayer ()
{
    if (this->pView)
    {
        delete this->pView;
    }
}

bool MusicPlayer::setup ()
{
    this->pView = new QQuickView ();
    this->pView->setColor (Qt::black);
    this->pView->setResizeMode (QQuickView::SizeViewToRootObject);
    this->pView->setFlags (static_cast<Qt::WindowFlags> (0x00800000));
    this->pView->setTitle (tr ("Music Player"));
    this->pView->rootContext ()->setContextProperty ("application", this);

    #ifdef ADD_MPRIS
        this->pView->rootContext ()->setContextProperty ("mprisPlayer", this->pMprisPlayer);
        this->pView->rootContext ()->setContextProperty ("ADD_MPRIS", QVariant (true));
    #else
        this->pView->rootContext ()->setContextProperty ("ADD_MPRIS", QVariant (false));
    #endif

    QUrl sSource (musicPlayerDirectory () + "/app/music-app.qml");
    this->pView->setSource (sSource);
    this->pView->show ();

    return true;
}
